from asc.core import env
from asc.core.ABC.APIRequestor import APIRequestor
from asc.exceptions import melhorenvio as exceptions_melhorenvio


class MelhorEnvio(APIRequestor):

    base_payload = {}
    base_api_url = env.get('melhorenvio__url_baseapi')
    base_headers = {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'User-Agent': f'{env.get("marketplace__soft_descriptor")} ('
                      f'{env.get("marketplace__soft_mail")})',
        'Authorization': ['Bearer', None]
    }

    auth_code = ''
    auth_config = {
        'client_id': env.get('melhorenvio__client_id'),
        'client_secret': env.get('melhorenvio__client_secret'),
        'redirect_uri': env.get('melhorenvio__url_callback')
    }
    auth_scope = [
        'cart-read',
        'cart-write','companies-read','companies-write','coupons-read',
        'coupons-write','notifications-read','orders-read','products-read',
        'products-write','purchases-read','shipping-calculate','shipping-cancel',
        'shipping-checkout','shipping-companies','shipping-generate','shipping-preview',
        'shipping-print','shipping-share','shipping-tracking','ecommerce-shipping',
        'transactions-read','users-read','users-write'
    ]

    def __init__(self, code: str = None, findOauth2: bool = True):
        super().__init__()

        self.base_payload = {
            **self.base_payload,
            **self.auth_config,
        }

        if code:
            self.auth_code = code
        elif findOauth2:
            self.auth_code = env.get('melhorenvio__code')

        if not self.auth_code: raise exceptions_melhorenvio.\
            AuthCodeIsRequired()

        self.set_oauth_code(self.auth_code)

    def __response(self, response, parseAs: str = 'json'):
        if parseAs == 'json':
            resp = response.json()

            if 'errors' in resp: raise Exception('ERROR MELHOR ENVIO API - ', resp)

            return resp

        return response.text

    def set_oauth_code(self,auth_code:str):
        """
        :param auth_code: código fornecido após autorização do cliente
        :return: Sucesso
        """
        self.auth_code = auth_code
        self.base_headers['Authorization'][1] = self.auth_code
        return True

    def GET_oauth_code(self):
        """
            Apenas gera o link para o cliente autorizar nosso aplicativo no cadastro do melhor envio.
            Este link envia o auth_code, devemos grava-lo para enviar o POST_oauth_token
            :return: str
        """

        url = f'oauth/authorize?client_id={self.auth_config.get("client_id")}' \
              f'&redirect_uri={self.auth_config.get("redirect_uri")}' \
              f'&response_type=code&state=teste&scope={"%20".join(self.auth_scope)}'
        """
        request = self.GET(
            url=url
        ).text
        """

        # Esta requisição não deve ser executada pela API, mas pelo cliente
        return self.mountURL(url)

    def GET_app_settings(self):

        return self.GET('api/v2/me/shipment/app-settings').json()

    def GET_services(self):
        """ Lista informações detalhadas de uma transportadora específica.
        """
        return self.GET('api/v2/me/shipment/services').json()

    def GET_order_search(self, **query):
        """ Busca informações de um pedido
        """

        return self.__response(
            self.GET('api/v2/me/orders/search', query=query)
        )

    def GET_shipment_tracking(self, cartId: str):
        """ Busca informações de um pedido
        """
        return self.__response(
            self.GET('api/v2/me/shipment/tracking', data={'orders': [cartId]})
        )

    def POST_oauth_token_refresh(self,refresh_token:str):
        """ Atualiza bearer token melhor envio.
        :param refresh_token: token recebido
        após autorização concedida por cliente.
        :return: JSON contendo resposta."""

        response = self.POST('oauth/token', data={
            'grant_type': 'refresh_token',
            'refresh_token': refresh_token,
            'client_id': self.auth_config.get('client_id'),
            'client_secret': self.auth_config.get('client_secret'),
        }).json()

        if 'error' in response: raise exceptions_melhorenvio.MelhorEnvioException(response)

        return response

    def POST_oauth_token(self):
        """
        :return: dict
        """
        req_data = {
            'grant_type': 'authorization_code',
            'code': self.auth_code,
            'client_id': self.auth_config.get('client_id'),
            'redirect_uri': self.auth_config.get('redirect_uri'),
            'client_secret': self.auth_config.get('client_secret')
        }

        response = self.POST('oauth/token', data=req_data).json()

        if 'error' in response: raise exceptions_melhorenvio.MelhorEnvioException(response)

        return response

    def POST_checkout(self, idCart: str):
        return self.__response(
            self.POST('api/v2/me/shipment/checkout', {
                'orders': [idCart]
            })
        )

    def POST_shipment_generate(self, idCart: str):
        return self.__response(
            self.POST('api/v2/me/shipment/generate', {
                'orders': [idCart]
            })
        )

    def POST_shipment_print(self, idCart: str):

        return self.__response(
            self.POST('api/v2/me/shipment/print', {
                'mode': 'private',
                'orders': [idCart]
            })
        )

    def GET_shipment_tracking(self, idCart: str):

        return self.__response(
            self.POST('api/v2/me/shipment/tracking', {
                'orders': [idCart]
            })
        )
